#include <QPixmapCache>
#include <QPixmap>
#include <QImage>
#include <QMatrix>
#include <QMatrix>

#include "flyer.h"
#include "options.h"
#include "gamelist.h"
#include "qmc2main.h"

// external global variables
extern MainWindow *qmc2MainWindow;
extern Options *qmc2Options;
extern Gamelist *qmc2Gamelist;
extern bool qmc2GuiReady;
extern bool qmc2ReloadActive;
extern bool qmc2ScaledFlyer;
extern bool qmc2SmoothScaling;
extern QTreeWidgetItem *qmc2CurrentItem;
extern QSettings *qmc2Config;

QPixmap *currentFlyerPixmap;

Flyer::Flyer(QWidget *parent)
  : QFrame(parent)
{
#ifdef QMC2_DEBUG
  qmc2MainWindow->log(LOG_QMC2, "DEBUG: Flyer::Flyer(QWidget *parent = 0x" + QString::number((ulong)parent, 16) + ")");
#endif

  QString ghostImagePath = qmc2Config->value("Frontend/FilesAndDirectories/DataDirectory", "data/").toString() + "img/ghost.png";
  qmc2GhostImagePixmap.load(ghostImagePath);
  setToolTip(tr("Game flyer image"));
  setStatusTip(tr("Game flyer image"));
}

Flyer::~Flyer()
{
#ifdef QMC2_DEBUG
  qmc2MainWindow->log(LOG_QMC2, "DEBUG: Flyer::~Flyer()");
#endif
}

void Flyer::paintEvent(QPaintEvent *e)
{
#ifdef QMC2_DEBUG
  qmc2MainWindow->log(LOG_QMC2, "DEBUG: Flyer::paintEvent(QPaintEvent *e = 0x" + QString::number((ulong)e, 16) + ")");
#endif

  if ( !qmc2CurrentItem )
    return;

  if ( qmc2CurrentItem->text(0) == tr("Waiting for data...") )
    return;

  QTreeWidgetItem *topLevelItem = qmc2CurrentItem;
  while ( topLevelItem->parent() )
    topLevelItem = topLevelItem->parent();

  QPainter p(this);

  QString gameName = topLevelItem->child(0)->text(1);
  static QPixmap cachedPixmap;

  if ( QPixmapCache::find("flyer" + gameName, cachedPixmap) ) {
    currentFlyerPixmap = &cachedPixmap;
  } else {
    qmc2CurrentItem = topLevelItem;
    loadFlyer(gameName);
  }

  if ( qmc2ScaledFlyer )
    drawScaledImage(currentFlyerPixmap, &p);
  else
    drawCenteredImage(currentFlyerPixmap, &p);
}

void Flyer::loadFlyer(QString gameName, QTreeWidgetItem *item)
{
#ifdef QMC2_DEBUG
  qmc2MainWindow->log(LOG_QMC2, "DEBUG: Flyer::loadFlyer(QString gameName = \"" + gameName + "\", QTreeWidgetItem *item = 0x" + QString::number((ulong)item, 16) + ")");
#endif

  if ( !item )
    item = qmc2CurrentItem;

  QString imagePath = qmc2Config->value("Frontend/FilesAndDirectories/FlyerDirectory", "data/fly/").toString() + gameName + ".png";
  static QPixmap pm;
  static QIcon nullIcon, ghostIcon(qmc2GhostImagePixmap);

  if ( pm.load(imagePath) ) {
    QPixmapCache::insert("flyer" + gameName, pm); 
    currentFlyerPixmap = &pm;
    item->setIcon(1, nullIcon);
  } else {
    currentFlyerPixmap = &qmc2GhostImagePixmap;
  }
}

void Flyer::drawCenteredImage(QPixmap *pm, QPainter *p)
{
#ifdef QMC2_DEBUG
  qmc2MainWindow->log(LOG_QMC2, "DEBUG: Flyer::drawCenteredImage(QPixmap *pm = 0x" + QString::number((ulong)pm, 16) + ", QPainter *p = 0x" + QString::number((ulong)p, 16) + ")");
#endif

  p->eraseRect(frameRect());

  if ( pm == NULL ) {
    p->end();
    return;
  }

  int posx = (frameRect().width() - pm->width()) / 2;
  int posy = (frameRect().height() - pm->height()) / 2;

  p->drawPixmap(posx, posy, *pm);
  p->end();
}

void Flyer::drawScaledImage(QPixmap *pm, QPainter *p)
{
#ifdef QMC2_DEBUG
  qmc2MainWindow->log(LOG_QMC2, "DEBUG: Flyer::drawScaledImage(QPixmap *pm = 0x" + QString::number((ulong)pm, 16) + ", QPainter *p = 0x" + QString::number((ulong)p, 16) + ")");
#endif

  if ( pm == NULL ) {
    p->eraseRect(frameRect());
    p->end();
    return;
  }

  double desired_width;
  double desired_height;

  if ( pm->width() > pm->height() ) {
    desired_width  = contentsRect().width();
    desired_height = (double)pm->height() * (desired_width / (double)pm->width());
    if ( desired_height > contentsRect().height() ) {
      desired_height = contentsRect().height();
      desired_width  = (double)pm->width() * (desired_height / (double)pm->height());
    }
  } else {
    desired_height = contentsRect().height();
    desired_width  = (double)pm->width() * (desired_height / (double)pm->height());
    if ( desired_width > contentsRect().width() ) {
      desired_width = contentsRect().width();
      desired_height = (double)pm->height() * (desired_width / (double)pm->width());
    }
  }

  QPixmap pmScaled;

  if ( qmc2SmoothScaling )
    pmScaled = pm->scaled(desired_width, desired_height, Qt::KeepAspectRatio, Qt::SmoothTransformation);
  else
    pmScaled = pm->scaled(desired_width, desired_height, Qt::KeepAspectRatio, Qt::FastTransformation);

  drawCenteredImage(&pmScaled, p);
}
